struct PixelToFrame
{
    float4 Color        : COLOR0;
};

float4x4 xWorldViewProjection;
float4x4 xLightWorldViewProjection;
float4x4 xWorld;
float4 xLightPos;
float xLightPower;
float xMaxDepth;


Texture xColoredTexture;

sampler ColoredTextureSampler = sampler_state { texture = <xColoredTexture> ; magfilter = LINEAR; minfilter = LINEAR; mipfilter=LINEAR; AddressU = mirror; AddressV = mirror;};


 Texture xShadowMap;

sampler ShadowMapSampler = sampler_state { texture = <xShadowMap> ; magfilter = LINEAR; minfilter = LINEAR; mipfilter=LINEAR; AddressU = clamp; AddressV = clamp;};



//------- Technique: ShadowMap --------

struct SMapVertexToPixel
{
    float4 Position     : POSITION;
    float3 Position2D    : TEXCOORD0;
};

struct SMapPixelToFrame
{
    float4 Color : COLOR0;
};

SMapVertexToPixel ShadowMapVertexShader( float4 inPos : POSITION)
{
    SMapVertexToPixel Output = (SMapVertexToPixel)0;

    Output.Position = mul(inPos, xLightWorldViewProjection);
    Output.Position2D = Output.Position;

    return Output;
}

SMapPixelToFrame ShadowMapPixelShader(SMapVertexToPixel PSIn)
{
    SMapPixelToFrame Output = (SMapPixelToFrame)0;

    Output.Color = PSIn.Position2D.z/xMaxDepth;

    return Output;
}

technique ShadowMap
{
    pass Pass0
{
        VertexShader = compile vs_2_0 ShadowMapVertexShader();
        PixelShader = compile ps_2_0 ShadowMapPixelShader();
    }
}


 //------- Technique: ShadowedScene --------
 
 struct SSceneVertexToPixel
 {
     float4 Position             : POSITION;
     float4 ShadowMapSamplingPos : TEXCOORD0;
 };
 
 struct SScenePixelToFrame
 {
     float4 Color : COLOR0;
 };
 
 SSceneVertexToPixel ShadowedSceneVertexShader( float4 inPos : POSITION)
 {
     SSceneVertexToPixel Output = (SSceneVertexToPixel)0;
 
     Output.Position = mul(inPos, xWorldViewProjection);
     Output.ShadowMapSamplingPos = mul(inPos, xLightWorldViewProjection);
 
     return Output;
 }
 
 SScenePixelToFrame ShadowedScenePixelShader(SSceneVertexToPixel PSIn)
 {
     SScenePixelToFrame Output = (SScenePixelToFrame)0;
 
     float2 ProjectedTexCoords;
     ProjectedTexCoords[0] = PSIn.ShadowMapSamplingPos.x/PSIn.ShadowMapSamplingPos.w/2.0f +0.5f;
     ProjectedTexCoords[1] = -PSIn.ShadowMapSamplingPos.y/PSIn.ShadowMapSamplingPos.w/2.0f +0.5f;
     Output.Color = tex2D(ShadowMapSampler, ProjectedTexCoords);
     // check if the coordinates are outside the shadow map
     if (ProjectedTexCoords[0]<0 || ProjectedTexCoords[0]>1)
		Output.Color.r=1;
     if (ProjectedTexCoords[1]<0 || ProjectedTexCoords[1]>1)
		Output.Color.r=1;
	
 
     return Output;
 }
 
 technique ShadowedScene
 {
     pass Pass0
     {
         VertexShader = compile vs_2_0 ShadowedSceneVertexShader();
         PixelShader = compile ps_2_0 ShadowedScenePixelShader();
     }
 }